<?php

declare(strict_types=1);

namespace Ig\IgGooglemaps\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Http\RequestFactory;
use TYPO3\CMS\Core\Http\JsonResponse;

class GeoCodeCommand extends Command
{
     
    protected $api_url = 'https://maps.googleapis.com/maps/api/geocode/json?';
    
    protected function configure()
    {
        $this
            ->setDescription('Geocode entries of tables')
            //->setHelp('')
            ->addArgument(
                'TableName',
                InputArgument::REQUIRED,
                'Tabellen Name'
            )
            
            ->addArgument(
                'Adress',
                InputArgument::OPTIONAL,
                'PLZ,ort,Strasse,StrassenNr,Land Spaltennamen in der Tabelle'
            )
            /*
              ->addArgument(
              'ids',
              InputArgument::OPTIONAL,
              'von id bis id (1,100)'
              )
            */
            ->addArgument(
                'maxvalues',
                InputArgument::OPTIONAL,
                'maximale anzahl an geocodes'
            )
            ->addOption(
                'latlng',
                'l',
                InputOption::VALUE_OPTIONAL,
                'latitude,longitude attribute names',
                'latitude,longitude'
            )

            ->addOption(
                'uid',
                'u',
                InputOption::VALUE_OPTIONAL,
                'primary key attribute name',
                'uid'
            );
    }


    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $extConf = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(
            \TYPO3\CMS\Core\Configuration\ExtensionConfiguration::class
        )->get('ig_googlemaps');
                                                          
        if(isset($extConf['googleServerGeocodingApiKey']) && $extConf['googleServerGeocodingApiKey'] != null){
            $key = $extConf['googleServerGeocodingApiKey'];
        }
        else{
            $io->error('ig_googlemaps API Key wurde nicht gefunden');
            return 1;
        }

        $tableName = $input->getArgument('TableName');
        $latLng = $input->getOption('latlng');
        $addressAttributeList = $input->getArgument('Adress') ?? 'zip, city, address, country';
        //$ids = $input->getArgument('ids');
        $maxvalues = intval($input->getArgument('maxvalues'));

        $uidAttribute = $input->getOption('uid') ?? 'uid';


        if(isset($latLng) && $latLng != null){
            list($latitudeAttribute, $longitudeAttribute) = GeneralUtility::trimExplode(',' ,$latLng);
        }
        else{
            $latitudeAttribute = 'latitude';
            $longitudeAttribute = 'longitude';
        }
        
        $addressAttributes = GeneralUtility::trimExplode(',' ,$addressAttributeList);
        /*
          if(isset($ids) && $ids != null){
          $ids = GeneralUtility::intExplode(',' ,$ids);
          }
          else{
          $ids = [
          '0',
          '1',
          ];
          }
        */
        
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($tableName);
            
        $queryBuilder
            ->select('*')
            ->from($tableName)
            ->where(
                /*
                  $queryBuilder->expr()->gte(
                  $uidAttribute,
                  $queryBuilder->createNamedParameter($ids[0], \PDO::PARAM_INT)
                  ),

                  $queryBuilder->expr()->lte(
                  $uidAttribute,
                  $queryBuilder->createNamedParameter($ids[1] ?? $ids[0], \PDO::PARAM_INT)
                  ),
                                    
                */
                                    
                $queryBuilder->expr()->orX(
                    $queryBuilder->expr()->isNull($latitudeAttribute),
                    $queryBuilder->expr()->neq(
                        $latitudeAttribute, 
                        $queryBuilder->createNamedParameter('')
                    ),
                    $queryBuilder->expr()->isNull($longitudeAttribute),
                    $queryBuilder->expr()->neq(
                        $longitudeAttribute, 
                        $queryBuilder->createNamedParameter('')
                    ),
                )
            );
        
        if ($maxvalues) {
            $queryBuilder->setMaxResults($maxvalues);
        }
        $result = $queryBuilder
                ->executeQuery()
                ->fetchAllAssociative();
        $entriesUpdate = 0;
        $entriesTotal = 0;
        foreach ($result as $row) {
            $entriesTotal++;
            $address = '';
            foreach($addressAttributes as $col){
                $address .= trim($row[$col]) . ' ';
            }
                        
            $queryData = http_build_query(
                [
                    'key'     => $key,
                    'address' => $address,
                ]
            );
            $url = $this->api_url . $queryData;
            $res = GeneralUtility::makeInstance(RequestFactory::class)->request($url, 'GET', ['headers' => ['accept' => 'application/json']]);
            $content = json_decode($res->getBody()->getContents(), true);

            if (isset($content['error_message'])) {                
                $io->error($content['error_message']);
                $io->error('used API key: ' . $key);
                return 1;
            }
            $location = $content['results'][0]['geometry']['location'];
            $latitude =  $location['lat'];
            $longitude = $location['lng'];
            $text = $row[$uidAttribute] . ': ' . $address;
            if ($latitude && $longitude) {
                $io->writeln($text . ' latLng: <fg=green>' . $latitude . ', ' . $longitude .'</>');
            } else {
                $io->writeln($text . ' latLng: <fg=red>NOT FOUND</>');
            }
                
            $row[$latitudeAttribute] = $latitude;
            $row[$longitudeAttribute] = $longitude;
            if($row[$latitudeAttribute] != null && $row[$longitudeAttribute] != null){
                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($tableName);
                $queryBuilder->update($tableName)
                             ->where(
                                 $queryBuilder->expr()->eq($uidAttribute, $queryBuilder->createNamedParameter($row[$uidAttribute], \PDO::PARAM_INT))
                             )
                             ->set($latitudeAttribute, $row[$latitudeAttribute])
                             ->set($longitudeAttribute, $row[$longitudeAttribute])
                             ->executeStatement();
                
                $entriesUpdate++;
            }
        }

        if ($entriesUpdate == $entriesTotal) {
            $io->success('Es wurden ' . $entriesUpdate . ' von ' . $entriesTotal . ' Einträge aktualisiert');
            return 0;
        } else {
            $io->warning('Es wurden ' . $entriesUpdate . ' von ' . $entriesTotal . ' Einträge aktualisiert');
            return 1;
        }
    }
}